// Lawson Labs, Inc.
// 15 Phoenixville Pike
// Malvern, PA  19355
// 800 321-5355 or 406 257-5355             August 30, 1996 version 3

#define STRICT
#include <windows.h>
#include <stdio.h>
#include <alloc.h>
#include <dos.h>
#include <stdlib.h>
#include <conio.h>
#include <math.h>
#include <ctype.h>
#include <string.h>
#include <time.h>
#ifdef __cplusplus
	 #define __CPPARGS ...
#else
	 #define __CPPARGS
#endif

#define BYTES_PER_LONG 4

//typedef for M201 struct
typedef struct{
	short		comPort;	  		//com port number 1-4
	double	scanInterval;	//scan interval in seconds
	short		numChannels;	//number of active channels for scanning
	short		tScan;			//type of scan 0 = single channel
									//					1 = normal
									//					2 = normal with calibration
	short		chan[6];			//control code array for scanning
	short		singleChan;    //single channel scanning channel
	short    lastDigInp; 	//last digital input value
	short 	version;			//Model 201 version
	short 	channel;			//currently selected channel (used in GetConversion)
	short 	subChan;			//currently selected sub channel (used in GetConversion)
	long	   rawCount;		//current raw count returned from Model 201
	double	voltage;			//current voltage
	int 	   digOutWord;		//value sent to Model 201 for digital output (0-255)
	short 	wordCnt;			//number of words in conversion (2 or 3)
	short 	biPolar;			//0 = unipolar  1 = bipolar
	int 	   errorType;		//error code returned from DLL
	short 	filter;			//0 = 4 Hz  1 = 40 Hz  2 = 400 Hz
	short 	gain;				//gain factor (powers of 2)
	float 	rate;				//sample rate (10 - 1027 Hz)
	short 	modeRegLo;		//mode registers for signon
	short 	modeRegMid;
	short 	modeRegHi;
	short		baud;				//selected baud rate code (0 - 5)
	short		sleeping;		//0 = Model 201 awake  1 = Model 201 asleep
	short		standBy;       //0 = Model 201 out of standby  1 = Model 201 in standby
	int		timeBaseCnt;	  //mode 202 specific element (reserved)
	unsigned int    average;  //remote averaging (powers of 2)
	//transmit/receive queue sizes as used by Windows COMM driver
	unsigned int    TXQUEUESIZE;
	unsigned int    RXQUEUESIZE;
}	T_M201;

//local prototypes

//make window the same size as the desktop
void    SetWindowToDesktop(void);

//turns on cursor in Win32 console mode
void    SetWin32CursorOn(void);

//turns off cursor in Win32 console mode
void    SetWin32CursorOff(void);

//read 201EWIN.CFG file from disk in current directory
short   ReadConfig(T_M201 far *M201,
						 short *localAvg);

//write 201EWIN.CFG file to disk in current directory
short   WriteConfig(T_M201 far *M201,
						  short *localAvg);

//select the scan interval
short   CalcScanTime(T_M201 far *M201);


//displays the scanning menu (returns 1 if set to start scanning)
short   ScanMenu(T_M201 far *M201);

//select the sampling rate for the A/D
void    SelectRate(T_M201 far *M201);

//   Get starting & ending control codes for a scanning channel
short   GetChannel(int value);

//display polled mode functions menu
void    PrintCommandMenu(T_M201 far *M201,
								 int Ch,
								 short localAvg,
								 short checkSum);

//perform selected function
void    DoCommand(T_M201 far *M201,
						int Ch,
						double *max,
						double *min,
						short *localAvg,
						short *checkSum);

//calculate standard deviation
void    StandardDev(T_M201 far *M201,
						  double *stDv,
						  double *rmsBits,
						  double *sdTot,
						  double *runningAvg,
						  double *sd,
						  int    *sdCount);

//prototypes for external dll functions

//This function will sign on to the Model 201 using the settings in M201
//If could not sign on, closes port and returns 0. Otherwise, returns 1,
//port is open and the Model 201 is active.
short  (FAR PASCAL *SignOn)(T_M201 far *M201);

//take a raw count and scale in volts
double (FAR PASCAL *Counts2Volts)(long counts,
											 short wordCnt,
											 short biBolar);

//Get a digital input from the Model 201.  Returns 0 if Model 201 did not
//echo back the Get Digital Input token.
short  (FAR PASCAL *GetDigIn)(T_M201 far *M201);

//Get the Model 201 version.  Returns 0 if Model 201 did not echo back the
//version token.
short  (FAR PASCAL *GetVer)(T_M201 far *M201);

//Send a digital output to the Model 201
short  (FAR PASCAL *SetDigOut)(int digOutWord);

//Set averaging in the Model 201 (powers of 2)
short  (FAR PASCAL *RemoteAvg)(T_M201 far *M201);

//Perform a full scale calibration on the currently selected channel
short  (FAR PASCAL *FullScaleCal)(T_M201 far *M201);

//Perform an offset calibration on the currently selected channel
short  (FAR PASCAL *OffsetCal)(T_M201 far *M201);

//Perform a system calibration (full scale on channel 6, offset on channel 7)
short  (FAR PASCAL *SysCal)(T_M201 far *M201);

//Get a conversion from the Model 201 on the currently selected channel
short  (FAR PASCAL *GetConversion)(T_M201 far *M201);

//Select channel and subchannel from M201.channel and M201.subChan
short  (FAR PASCAL *SetChanCode)(T_M201 far *M201);

//Start scanning using selected scan type. Returns 0 if scan token is not
//echoed back.
short  (FAR PASCAL *StartScanning)(T_M201 far *M201);

//Get one scan from currently selected comm port buffer. It will wait the
//calculated time for the scan to come in. Returns 0 if failed. Returns type
//of failure in M201.errorType.
//M201.errorType = 1		Start scan framing failure
//M201.errorType = 2		not enough chars, possible communications error
//M201.errorType = 3		Comm port buffer overflow
//M201.errorType = 0xA  Scan rate too fast for this scan type
short  (FAR PASCAL *GetOneScan)(T_M201 far *M201, long far *clientDataBuffer);

//Tell the Model 201 to stop scanning. Returns 0 if Stop scan token is not
//echoed back
short   (FAR PASCAL *StopScanning)(T_M201 far *M201);

//Get counts from ScanBuffer, scale them in volts, and remove counts from
//ScanBuffer
void   (FAR PASCAL *Gconv)(T_M201 far *M201,
									long far *ScanBuffer);

//Remove size elements from ScanBuffer
void   (FAR PASCAL *RemoveFromBuffer)(int size,
												  long far *ScanBuffer);

//Set the filter  0 = 4 Hz   1 = 40 Hz   2 = 400 Hz
void   (FAR PASCAL *SetFilter)(short filter);

//Send mode registers to Model 201. Returns 0 if the registers read back
//incorrectly
short  (FAR PASCAL *SendADMode)(T_M201 far *M201);

//Get and compare Model 201 checksum. Returns 0 if failed, 1 if checksum is OK
short  (FAR PASCAL *GetCheckSum)(void);

//Send the stop scan command to stop a scan if scanning, send master reset,
//and close the currently selected comm port.
short  (FAR PASCAL *CloseDevice)(void);

//Put the Model 201 in sleep mode
short  (FAR PASCAL *GotoSleep)(T_M201 far *M201);

//Wait the filter settling time
void   (FAR PASCAL *FilterDelay)(short filter);

//Check the selected scan interval to see if it is in limits
short  (FAR PASCAL *CheckScanTime)(T_M201 far *M201,
											  double *maxTime,
											  double *minTime);


//The next two functions only need to be called if handling more than one
//Model 201 at a time.

//Get the comm handle after Signon
#ifdef WIN32
void   (FAR PASCAL *GetComHandle)(HANDLE *idComDev);
#else
void   (FAR PASCAL *GetComHandle)(int *idComDev);
#endif

//Set the comm handle before CloseDevice
#ifdef WIN32
void   (FAR PASCAL *SetComHandle)(HANDLE idComDev);
#else
void   (FAR PASCAL *SetComHandle)(int idComDev);
#endif

//Global Variables
HINSTANCE 	hLibrary;

#ifdef WIN32
static HANDLE idComDev;   //  device channel to com port as used by Windows
#else
static int idComDev;   //  device channel to com port as used by Windows
#endif

int main(void)
{
T_M201 					M201;
FILE 	       			*out;

long far 				*ScanBuffer;
HANDLE 		         hglbScanBuff;

long 						longBuffSize, rawCount, scanCnt;
int 						i,n,p_row = 14,logging, startCC, endCC, ccode,Ch;
double 					voltage, max=-12000, min=12000;
char 						fileName[12], tmpbuf[80];;
short						receiveError, localAvg=1, checkSum = 1;
short						cc[6], tempCh, tempSub;
short 					bauds[]={  //  table for baud rate codes
							/* 0 */  9600,
							/* 1 */  4800,
							/* 2 */  2400,
							/* 3 */  1200,
							/* 4 */   600,
							/* 5 */   300};
//variables for standard deviation calculation
double               stDv=0, rmsBits=0, sdTot = 0, runningAvg=0, sd[41];;
int    					sdCount=1;

	M201.timeBaseCnt = -1;		//model 202 specific element (reserved)

	SetWindowToDesktop();

	if (!ReadConfig(&M201, &localAvg))
	{
	  gotoxy(24,10);
	  printf("unable to read 201EWIN.CFG file");
	  gotoxy(26,11);
	  printf("press any key to continue");
	  getch();
	}
	M201.standBy = 0;				//set to not in standby
	M201.sleeping = 0;         //set to awake

	longBuffSize = 1000;
	hglbScanBuff = GlobalAlloc(GMEM_FIXED,(longBuffSize * BYTES_PER_LONG));
	ScanBuffer = (long far *)GlobalLock(hglbScanBuff);

#ifdef WIN32
	hLibrary = LoadLibrary ("24BIT32.DLL");
#else
	hLibrary = LoadLibrary ("24BIT16.DLL");
#endif

	(FARPROC) SignOn =GetProcAddress(hLibrary, "SignOn");
	(FARPROC) GetDigIn =GetProcAddress(hLibrary, "GetDigIn");
	(FARPROC) GetVer =GetProcAddress(hLibrary, "GetVer");
	(FARPROC) SetDigOut =GetProcAddress(hLibrary, "SetDigOut");
	(FARPROC) RemoteAvg =GetProcAddress(hLibrary, "RemoteAvg");
	(FARPROC) FullScaleCal =GetProcAddress(hLibrary, "FullScaleCal");
	(FARPROC) OffsetCal =GetProcAddress(hLibrary, "OffsetCal");
	(FARPROC) SysCal =GetProcAddress(hLibrary, "SysCal");
	(FARPROC) Counts2Volts =GetProcAddress(hLibrary, "Counts2Volts");
	(FARPROC) GetConversion =GetProcAddress(hLibrary, "GetConversion");
	(FARPROC) SetChanCode=GetProcAddress(hLibrary, "SetChanCode");
	(FARPROC) StartScanning=GetProcAddress(hLibrary, "StartScanning");
	(FARPROC) GetOneScan=GetProcAddress(hLibrary, "GetOneScan");
	(FARPROC) StopScanning=GetProcAddress(hLibrary, "StopScanning");
	(FARPROC) Gconv=GetProcAddress(hLibrary, "Gconv");
	(FARPROC) RemoveFromBuffer=GetProcAddress(hLibrary, "RemoveFromBuffer");
	(FARPROC) SetFilter=GetProcAddress(hLibrary, "SetFilter");
	(FARPROC) SendADMode=GetProcAddress(hLibrary, "SendADMode");
	(FARPROC) GetCheckSum=GetProcAddress(hLibrary, "GetCheckSum");
	(FARPROC) CloseDevice=GetProcAddress(hLibrary, "CloseDevice");
	(FARPROC) GotoSleep=GetProcAddress(hLibrary, "GotoSleep");
	(FARPROC) FilterDelay=GetProcAddress(hLibrary, "FilterDelay");
	(FARPROC) CheckScanTime=GetProcAddress(hLibrary, "CheckScanTime");
	(FARPROC) GetComHandle=GetProcAddress(hLibrary, "GetComHandle");
	(FARPROC) SetComHandle=GetProcAddress(hLibrary, "SetComHandle");

TOP_MENU:

  Ch = 0;

  while(Ch != 27)
  {
	 clrscr();
	 gotoxy(22,4);
	 printf ("MODEL 201 Data Acquisition System");
	 gotoxy(34,6);
	 printf ("Main Menu");
	 gotoxy(20,8);
	 printf ("1) Baud rate          %4d",bauds[M201.baud]);
	 gotoxy(20,10);
	 printf ("2) Serial port        COM%d",M201.comPort+1);
	 gotoxy(20,12);
	 printf ("3) Input buffer     %6d bytes",M201.RXQUEUESIZE);
	 gotoxy(20,14);
	 printf ("4) Polled Mode Operation");
	 gotoxy(20,16);
	 printf ("5) Scanning Mode Operation");
	 gotoxy(18,18);
	 printf ("Esc) Exit program");

	 if (M201.sleeping)
	 {
		gotoxy(20,20);
		printf("Model 201 is in sleep mode");
	 }

	 Ch = getch();

	 if (Ch == 27)
	 {
		goto EXIT_POINT;
	 }

	 if (Ch == '1')
	 {

		if (M201.baud++ == 5)
		{
		  M201.baud = 0;
		}

	 }

	 if (Ch == '2')
	 {

		if (M201.comPort++ == 3)
		{
		  M201.comPort = 0;
		}

	 }

	 if (Ch == '3')
	 {
		clrscr();
		SetWin32CursorOn();

		gotoxy (14,12);
		printf("(enter between 11 and 32767)");
		gotoxy(15,10);
		printf("Enter size of input buffer  ");
		#ifdef WIN32
		getch();    /////.new.
		#endif
		fgets(tmpbuf,80,stdin);
		sscanf(tmpbuf,"%d",&M201.RXQUEUESIZE);
		SetWin32CursorOff();

		if (M201.RXQUEUESIZE < 11)
		{
			M201.RXQUEUESIZE = 11;
		}

	 }

	 if (Ch == '4')
	 {
		M201.sleeping = 0;
		clrscr();
		gotoxy(30,12);
		printf("Signing on...");

		if (!SignOn(&M201))
		{
		  gotoxy(16,14);
		  printf ("unable to sign on.  Press any key to continue");
		  getch();
		  goto TOP_MENU;
		}

		GetComHandle(&idComDev);		//get handle to comm port

		gotoxy(20,12);
		printf("Performing system calibration...");
		SysCal(&M201);

		GetVer(&M201);				//get version
		RemoteAvg(&M201);       //set averaging
		SetChanCode(&M201);     //set channel and subchannel

		FilterDelay(M201.filter);	//wait filter settling time

		goto POLLED;
	 }

	 if (Ch == '5')
	 {
		clrscr();
		goto GET_SCANS;
	 }

  } //while Ch != 27

goto EXIT_POINT;

POLLED:

  clrscr();
  PrintCommandMenu(&M201, Ch, localAvg, checkSum);
  while (Ch != 27)		//repeat until esc key pressed or an error
  {

	 while (!kbhit() && !M201.standBy)//loop until keyboard hit
	 {
		gotoxy(1,p_row++);

		voltage = 0;
		rawCount = 0;

		for (i=0;i<localAvg;i++)
		{
		  if(!GetConversion(&M201))
		  {
			 gotoxy(1,23);
			 printf("get conversion failed, press any key to continue");
			 printf("\n       %X       ",M201.errorType);
			 while (Ch != 27)
			 {
				Ch = getch();
			 }

			 goto TOP_MENU;
		  }

		  voltage += M201.voltage;
		  rawCount += M201.rawCount;
		}
		M201.voltage = voltage/localAvg;
		M201.rawCount = rawCount/localAvg;

		printf ("   %9ld     %9.3lf",M201.rawCount,M201.voltage);

		if (max < M201.voltage)
		{
		  max = M201.voltage;               //update maximum
		}

		if (min > M201.voltage)
		{
		  min = M201.voltage;               //update minimum
		}

		StandardDev(&M201,
						&stDv,
						&rmsBits,
						&sdTot,
						&runningAvg,
						sd,
						&sdCount);

		if (p_row == 23)
		{
		  p_row = 14;
		  gotoxy(40,14);
		  printf ("Maximum     %9.3lf mv       ", max);
		  gotoxy (40,15);
		  printf ("Minimum     %9.3lf mv       ", min);
		  gotoxy (40,16);
		  printf ("Delta       %9.3lf mv       ", fabs(max - min));
		  gotoxy (40,17);
		  printf ("Average     %9.3lf mv       ", runningAvg);
		  gotoxy (40,19);
		  printf ("Standard deviation  %9.3lf mv    ", stDv);
		  gotoxy (40,20);
		  printf ("Effective Resolution  %5.1lf   bits", rmsBits);
		  gotoxy (1,14);
		}

	 }   //while (!kbhit()  && !M201.standBy)

	 Ch = getch();

	 if (Ch != 27)
	 {

		if (islower(Ch))
		{
		  Ch = toupper(Ch);
		}

		if (Ch == 'C' || Ch == 'E' || Ch == 'G' || Ch == 'P' || Ch == 'W')
		{
		  stDv=0; rmsBits=0; sdTot = 0; runningAvg=0; sdCount=1;
		}

		DoCommand(&M201, Ch, &max, &min, &localAvg, &checkSum);

	 }

	 if (M201.sleeping)
	 {
		goto TOP_MENU;
	 }

  }	//while (Ch != 27)

  goto TOP_MENU;  //end of polled mode example


GET_SCANS:       //start of scanned mode example

  tempCh = M201.channel;		//hold values for current channel and subchannel
  tempSub = M201.subChan;

  if (!ScanMenu(&M201))		   //setup scanning details
  {
	 goto TOP_MENU;
  }

  M201.sleeping = 0;
  clrscr();
  gotoxy(30,12);
  printf("Signing on...");

  if (!SignOn(&M201))         //signon again with new scanning details
  {
	 gotoxy(16,14);
	 printf ("unable to sign on.  Press any key to continue");
	 getch();
	 goto TOP_MENU;
  }

  GetComHandle(&idComDev);		//get handle to comm port

  gotoxy(20,12);
  printf("Performing system calibration...");
  SysCal(&M201);

  RemoteAvg(&M201);

  clrscr();

  logging = 0;

  SetWin32CursorOn();
  gotoxy(20,13);
  printf("Enter name for disk file,");
  gotoxy(20,15);
  printf("or just enter for no file ");
  #ifdef WIN32
  getch();    /////.new.
  #endif
  gets(fileName);
  SetWin32CursorOff();

  if (strlen(fileName))
  {
	 logging = 1;
  }

  if(logging)
  {
	 if ((out = fopen(fileName, "at"))== NULL)
	 {
		gotoxy(20,19);
		printf("Could not open/create file\n");
		goto END_SCANS;
	 }
  }

  M201.numChannels = 0;

  for (n= 0;n<=5;n++)    //initialize control code array
  {
	 cc[n] = fmod((double)M201.chan[n],16) - (M201.chan[n]>>4) + 1;
	 M201.numChannels += cc[n];  //figure total number of channels/scan
  }

  clrscr();

  scanCnt = 0;

  if (M201.tScan == 0)
  {
	 ccode = M201.subChan + 3;				//setup location vars for display of
	 n = M201.channel * 12 + 5;         //single channel voltages
	 SetChanCode(&M201);                //set control code for single channel scanning
  }

  StartScanning(&M201);  	//Start scanning using selected scan type

  clrscr();
  printf(" CC    chan 0      chan 1      chan 2     ");
  printf(" chan 3      chan 4      chan 5\n\n");
  for(i=0;i<=15;i++)
  {
	 printf(" %d\n",i);
  }
  gotoxy(2,25);
  printf("Wait               Press any key to quit    ");
  printf("Scan interval %10.3lf secs",M201.scanInterval);

SCANSTATUS:

  while(!kbhit())		//while no keystroke
  {

	 receiveError = GetOneScan(&M201, ScanBuffer);	//get one scan from comm
																	//port buffer
	 if (receiveError != 0)
	 {
		gotoxy(2,25);
		printf("SCAN # %ld", ++scanCnt);

		if(M201.tScan == 0)
		{
		  gotoxy(n,ccode);
		  Gconv(&M201, ScanBuffer);		//get voltage
		  printf("%9.3lf", M201.voltage);
		  if (logging)
		  {
			 fprintf(out,"%9.3lf  ", M201.voltage);
		  }

		  if (logging)
		  {
			 fprintf(out,"\n");
		  }

		  goto SCANSTATUS;
		}

		if(M201.tScan == 2)
		{
		  gotoxy(2,23);
		  printf("Offset cal ");
		  Gconv(&M201, ScanBuffer);		//get voltage
		  printf("   %9.3lf", M201.voltage);

		  if (logging)
		  {
			 fprintf(out,"%9.3lf  ", M201.voltage);
		  }

		  gotoxy(27,23);
		  printf("Full scale cal ");
		  Gconv(&M201, ScanBuffer);		//get voltage
		  printf("   %9.3lf", M201.voltage);

		  if (logging)
		  {
			 fprintf(out,"%9.3lf  ", M201.voltage);
		  }
		}

		for(n=0;n<=5;n++)			//display channel voltages
		{
		  startCC = (int)(M201.chan[n] / 16);
		  endCC = cc[n] + startCC - 1;

		  for(ccode=startCC;ccode<=endCC;ccode++)
		  {
			 gotoxy((n * 12) + 5,(ccode + 3));
			 Gconv(&M201, ScanBuffer);		//get voltage
			 printf("%9.3lf", M201.voltage);

			 if (logging)
			 {
				fprintf(out,"%9.3lf  ", M201.voltage);
			 }

		  }

		}

		if (logging)
		{
		  fprintf(out,"\n");
		}

		if(ScanBuffer[0] != 0x0F)        // should be end of scan token
		{
		  printf("End scan framing failure ");
		  getch();
		  goto END_SCANS;
		}

		RemoveFromBuffer(1,ScanBuffer);		//remove end scan token
	 }
	 else
	 {

		gotoxy(2,19);
		if (M201.errorType == 1)
		{
		  printf("Start scan framing failure\n");
		}

		if (M201.errorType == 2)
		{
		  printf("not enough chars, possible communications error");
		}

		if (M201.errorType == 3)
		{
		  printf("buffer overflow");
		}

		if (M201.errorType == 0xA)
		{
		  printf("scan rate too fast for this scan type");
		}

		printf("\n press any key to continue");
		getch();

		goto END_SCANS;
	 }

  }             //end while !kbhit()
END_SCANS:
  if (logging)
  {
	 fclose(out);
  }

 gotoxy(1,25);
 printf("                    Wait                 ");

  while(kbhit())		//empty keyboard buffer
  {
	 getch();
  }


 StopScanning(&M201);		//Tell the Model 201 to stop scanning

 M201.channel = tempCh;		  		//restore channel and subchannel
 M201.subChan = tempSub;

 goto GET_SCANS;		//end of scanning example

EXIT_POINT:

	SetComHandle(idComDev);		//set handle to current comm port
	CloseDevice();					//close comm port
	GlobalFree(hglbScanBuff);
	GlobalUnlock(hglbScanBuff);
	FreeLibrary(hLibrary);

	clrscr();
	printf("\nDo you wish to save configuration (y/n)?  ");
	Ch = getch();

	if (Ch == 'Y' || Ch == 'y')
	{

	  WriteConfig(&M201,&localAvg);
	}

	SetWin32CursorOn();
	DestroyWindow(GetActiveWindow());
	exit(0);
	return 0;

}

//   Get starting & ending control codes for a scanning channel
short GetChannel(int value)
{
short startCC=-1, endCC=-1, channel;
static char  tmpbuf1[80];

	while ((startCC < 0) || (startCC > 15))
	{
	  clrscr();
	  gotoxy(4,23);
	  printf( "Enter an end code smaller than the start code to skip this channel");

	  SetWin32CursorOn();
	  gotoxy(1,12);
	  printf("Channel number:    %d\n",value);
	  printf("Enter starting control code  ");
	  #ifdef WIN32
	  getch();    /////.new.
	  #endif

	  tmpbuf1[1] = 0;
	  fgets(tmpbuf1,80,stdin);

	  if (tmpbuf1[1] >= '0')
	  {
		 startCC =  (short)((tmpbuf1[0] - '0')*10);
		 startCC += (short)(tmpbuf1[1] - '0');
	  }
	  else
	  {
		 startCC = (short)(tmpbuf1[0] - '0');
	  }

	}

	while ((endCC < 0) || (endCC > 15))
	{
	  printf("Enter ending control code  ");

	  #ifdef WIN32
	  getch();    /////.new.
	  #endif
	  tmpbuf1[1] = 0;
	  fgets(tmpbuf1,80,stdin);
	  SetWin32CursorOff();

	  if (tmpbuf1[1] >= '0')
	  {
		 endCC =  (short)((tmpbuf1[0] - '0')*10);
		 endCC += (short)(tmpbuf1[1] - '0');
	  }
	  else
	  {
		 endCC = (short)(tmpbuf1[0] - '0');
	  }

	}

	if ( startCC > endCC)
	{
		startCC = 1;
		endCC = 0;
	}

	channel = (short)(startCC * 16 + endCC);
	return channel;
}


void PrintCommandMenu(T_M201 far *M201,	//display polled mode function menu
							 int Ch,
							 short localAvg,
							 short checkSum)
{
int nn;

  gotoxy (10,1);
  printf ("POLLED  MENU   Lawson Labs Model 201   Version=%d\n", M201->version);
  gotoxy (5,3);
  printf ("C) A/D input Channel    %d", M201->channel);
  gotoxy (5,4);
  printf ("E) External sub-channel %d", M201->subChan);

  if (M201->subChan < 10)
  {
	 printf (" ");
  }

  gotoxy (5,5);
  printf ("A) remote Averaging     %d", M201->average);
  gotoxy (5,6);
  printf ("L) Local averaging      %d        ", localAvg);
  gotoxy (5,7);

  printf ("T) filTer             ");
  if (M201->filter == 0)
  {
	 printf ("  4 hz");
  }
  else if (M201->filter == 1)
  {
	 printf (" 40 hz ");
  }
  else
  {
	 printf ("400 hz ");
  }

  gotoxy (5,8);
  printf ("G) Gain factor        %3d",M201->gain);

  gotoxy (5,9);
  printf ("R) sample Rate       %6.1lf",M201->rate);

  gotoxy (5,10);
  printf( "B) ");
  if (M201->standBy)
  {
	 printf("in standBy mode    ");
  }
  else
  {
	 printf("out of standBy mode");
  }

  gotoxy (43,3); printf ("D) Digital inputs ");

  if (Ch == 'D')
  {
	 printf ("  %d",M201->lastDigInp);

	 if (M201->lastDigInp < 100)
	 {
		printf ("  ");
	 }

	 gotoxy (61,2); printf ("Decimal 76543210\n");
	 gotoxy (69,3);

	 for (nn = 7;nn != -1; nn--)
	 {

		if (M201->lastDigInp&(int)(pow(2, nn)))
		{
		  printf ("1");
		}
		else
		{
		  printf ("0");
		}

	 }

  }

  gotoxy (43,4); printf ("U) digital oUtput   ");

  if (Ch == 'U')
  {
	 printf ("%d  ",M201->digOutWord);
	 Ch = 0;
  }

  gotoxy (43,5); printf ("K) checKsum         ");

  if (Ch == 'K')
  {
	 if (checkSum)
	 {
		printf ("     OK");
	 }
	 else
	 {
		printf (" not OK");
	 }

  }
  else
  {
	 printf ("        ");
  }

  gotoxy (43,6);
  printf ("S) System calibration");
  gotoxy (43,7);
  printf ("O) Offset calibration");

  gotoxy (43,8);
  printf ("W) Word length      ");

  if (M201->wordCnt == 3)
  {
	 printf("24 bits");
  }
  else
  {
	 printf("16 bits");
  }

  gotoxy (43,9);
  printf ("P) Polarity        ");

  if (M201->biPolar)
  {
	 printf (" Bipolar");
  }
  else
  {
	 printf ("Unipolar");
  }

  gotoxy (43,10);
  printf ("Z) sleep");

  gotoxy (6,12); printf ("Counts      Millivolts\n");
  gotoxy (1,24);
  printf (" Press ESC for menu or space to reset max/min                       ");
}


void    StandardDev(T_M201 far *M201,		//calculate standard deviation
						  double *stDv,
						  double *rmsBits,
						  double *sdTot,
						  double *runningAvg,
						  double *sd,
						  int    *sdCount)
{//standard deviation and RMS resolution calculation
int i = 0, nnn,sdGrp=40, bits;
double tempStDv;

  sd[*sdCount] = M201->rawCount;
  (*sdTot) += M201->rawCount;

  if ((*sdCount) == sdGrp)
  {
	 (*stDv) = 0;
	 (*sdTot) = (*sdTot) / sdGrp;

	 for (nnn = 1; nnn != sdGrp; nnn++)
	 {
		(*stDv) = (*stDv) + ((*sdTot) - sd[nnn])*((*sdTot) - sd[nnn]);
	 }

	 (*stDv) = sqrt((*stDv) / (sdGrp-1));
	 tempStDv = (*stDv);

CALCNEXT:

	 if (tempStDv >= 1)
	 {
		i = i + 1;
		tempStDv = tempStDv / 2;
		goto CALCNEXT;
	 }

	 if (M201->wordCnt == 3)
	 {
		bits = 24;
	 }
	 else
	 {
		bits = 16;
	 }

	 (*rmsBits) = bits - i;

	 if ((tempStDv < .75) && (i != 0))
	 {
		(*rmsBits) = (*rmsBits) + .5;
	 }

	 (*runningAvg) = Counts2Volts((long)(*sdTot),
											 M201->wordCnt,
											 M201->biPolar);

	 (*stDv) = Counts2Volts((long)(*stDv),
											 M201->wordCnt,
											 M201->biPolar);

	  if (M201->biPolar)
	  {
		 (*stDv) += 5000;		//unipolar voltage range
	  }

	 (*sdTot) = 0; (*sdCount) = 1;
  }
  else
  {
	 (*sdCount)++;
  }

}

void DoCommand(T_M201 far *M201,		//perform selected function
					int Ch,
					double *max,
					double *min,
					short *localAvg,
					short *checkSum)
{
char tempCh=0;
short tempSubChan, tempGain;
char tmpbuf[80];

  if (islower(Ch))
  {
	 Ch = toupper(Ch);
  }

  gotoxy (1,24);
  printf ("                                                   ");

  if (Ch == 'Z')
  {
	 M201->sleeping = GotoSleep(M201);
  }

  if (Ch == 'T')
  {

	 if(++M201->filter == 3)
	 {
		M201->filter = 0;
	 }

	 SetFilter(M201->filter);
  }

  if (Ch == 'R')
  {
	 SelectRate(M201);
  }

  if (Ch == 'B')
  {

	 if(M201->standBy)
	 {
		M201->standBy = 0;
	 }
	 else
	 {
		M201->standBy = 1;
	 }

	 SendADMode(M201);
  }

  if (Ch == 'G')
  {
	 M201->gain = (short)fmod(M201->gain*2,255);
	 SendADMode(M201);
  }

  if (Ch == 'P')
  {

	 if(M201->biPolar)
	 {
		M201->biPolar = 0;
	 }
	 else
	 {
		M201->biPolar = 1;
	 }

	 SendADMode(M201);

	 SysCal(M201);

	 SetChanCode(M201);

  }

  if (Ch == 'W')
  {
	 if(M201->wordCnt == 2)
	 {
		M201->wordCnt = 3;
	 }
	 else
	 {
		M201->wordCnt = 2;
	 }

	 SendADMode(M201);
  }

  if (Ch == 'A')
  {

	 while((tempCh != 27) && (tempCh != 13))
	 {

		while (!kbhit())
		{
		  gotoxy (1,24);
		  printf ( "Press + increase, - decrease averaging, Enter to continue ");
		  printf ( "%6d", M201->average);
		}

		tempCh =(short) getch();
		if (tempCh == '+')
		{
		  M201->average *= 2;
		}

		if (tempCh == '-')
		{
		  M201->average /= 2;
		}

		if (M201->average  < 1)
		{
		  M201->average = 32678;
		}
		else if (M201->average > 32678)
		{
		  M201->average = 1;
		}

	 }

	 RemoteAvg(M201);
  }

  if (Ch == 'C')
  {
	 tempCh = -1;

	 while ((tempCh < '0') || (tempCh > '7'))
	 {
		gotoxy(1,24);
		printf("Enter channel ");
		tempCh =(short)getche();
	 }

	 M201->channel = (short)(tempCh - '0');

	 SetChanCode(M201);
	 gotoxy(1,24);
    printf("                       waiting filter settling time...");
	 FilterDelay(M201->filter);

  }

  if (Ch == 'D')
  {
	 GetDigIn(M201);
  }

  if (Ch == 'E')
  {
	 tempSubChan = -1;

	 while ((tempSubChan < 0) || (tempSubChan > 15))
	 {
	   SetWin32CursorOn();
		gotoxy(1,24);
		printf("Enter sub channel ");
		#ifdef WIN32
		getch();    /////.new.
		#endif
		tmpbuf[1] = 0;
		fgets(tmpbuf,80,stdin);
		SetWin32CursorOff();

		if (tmpbuf[1] >= '0')
		{
		  tempSubChan =  (short)((tmpbuf[0] - '0')*10);
		  tempSubChan += (short)(tmpbuf[1] - '0');
		}
		else
		{
		  tempSubChan = (short)(tmpbuf[0] - '0');
		}

	 }

	 M201->subChan = tempSubChan;

	 SetChanCode(M201);

	 gotoxy(1,24);
    printf("                       waiting filter settling time...");
	 FilterDelay(M201->filter);

  }

  if (Ch == 'L')
  {
	 (*localAvg) = 0;
	 SetWin32CursorOn();

	 while ((*localAvg) < 1)
	 {
		gotoxy(1,24);
		printf("Enter local averaging ");
		#ifdef WIN32
		getch();    /////.new.
		#endif
		fgets(tmpbuf,80,stdin);
		sscanf(tmpbuf,"%d",localAvg);
	 }

	 SetWin32CursorOff();

  }

  if (Ch == 'K')
  {
	 (*checkSum) = GetCheckSum();
  }

  if (Ch == 'S')
  {
	 gotoxy(31,24);
	 printf("Calibrating...");
	 tempCh =M201->channel;
	 tempGain = M201->gain;
	 if (M201->gain != 1)
	 {
		M201->gain = 1;
		SendADMode(M201);
	 }

	 SysCal(M201);

	 if (tempGain != 1)
	 {
		M201->gain = tempGain;
		SendADMode(M201);

		M201->channel = 7;
		OffsetCal(M201);
	 }

	 M201->channel=tempCh;
	 SetChanCode(M201);

	 FilterDelay(M201->filter);

  }

  if (Ch == 'O')
  {
	 gotoxy(31,24);
	 printf("Calibrating...");
	 OffsetCal(M201);
  }

  if (Ch == 'F')
  {
	 gotoxy(31,24);
	 printf("Calibrating...");
	 FullScaleCal(M201);
  }

  if (Ch == 'U')
  {
	 M201->digOutWord = -1;
	 SetWin32CursorOn();

	 while ((M201->digOutWord < 0) || (M201->digOutWord > 255))
	 {
		gotoxy(1,24);
		printf ("Enter Output Code ");
		#ifdef WIN32
		getch();    /////.new.
		#endif
		fgets(tmpbuf,80,stdin);
		sscanf(tmpbuf,"%d",&M201->digOutWord);
	 }

	 SetWin32CursorOff();
	 SetDigOut(M201->digOutWord);
  }

  if (Ch == ' ')
  {
	 (*max) = -12000.0;
	 (*min) = 12000.0;   //reset max-min
  }

  if(!M201->sleeping)
  {
	 PrintCommandMenu(M201, Ch, (*localAvg), (*checkSum));
  }

}


short ScanMenu(T_M201 far *M201)	//displays the scanning menu (returns 1 if
											//set to start scanning
{
int n, vala;
double maxTime, minTime;
short	cc[6], i;
int Ch;
char tmpbuf[80];

SCANMENU1:
  clrscr();
  M201->numChannels = 0;

  for (n= 0;n<=5;n++)    //initialize control code array
  {
	 cc[n] = fmod((double)M201->chan[n],16) - (M201->chan[n]>>4) + 1;
	 M201->numChannels += cc[n];  //figure total number of channels/scan
  }

  gotoxy(27,1);
  printf("SCANNING  CONFIGURATION");
  gotoxy(14,3);
  printf("CHANNEL      EXTERNAL CONTROL CODE      NUMBER OF");
  gotoxy(54,4);
  printf("DATA POINTS");

  if(M201->tScan == 0)
  {
	 M201->subChan = (short)(M201->singleChan&15);
	 M201->channel = (short)(M201->singleChan>>4);
	 gotoxy(18,7);
	 printf ("%d",M201->channel);
	 printf("               %d                 1", M201->subChan);
	 gotoxy(15,10);
	 printf("   PRESS 0 - 5 FOR CHANNEL");
  }
  else
  {
	 gotoxy(29,5);
	 printf("FROM           TO        THIS CHANNEL");

	 for (n=0;n<=5;n++)
	 {
		gotoxy(17,7 + n * 2);
		printf("%d)           ", n);
		printf( "%2d            %2d           %2d",M201->chan[n]>>4, (int)fmod((double)M201->chan[n],16), cc[n]);
	 }
  }

  gotoxy(17,19);
  printf("6) Scan Interval  %11.4lf seconds",M201->scanInterval);

  if((M201->gain != 1) && (M201->tScan == 2))
  {
	 M201->tScan = 0;    //no cal scan unless gain=1
  }

  gotoxy(17,21);
  printf("7) Scan type: ");

  if (M201->tScan == 0)
  {
	 printf ("Single Channel  ");
  }
  else if (M201->tScan == 1)
  {
	 printf ("Normal          ");
  }
  else
  {
	 printf ("Self-Calibration");
  }

  gotoxy(11,23);
  printf("  Enter) Perform the scan");
  gotoxy(15,25);
  printf("Esc) To continue");

  if(M201->tScan > 0)
  {
	 printf("       data points per scan  %d",M201->numChannels);
  }

  Ch = getch();
  vala = Ch -'0';

  if((vala == 0) && (Ch != '0'))
  {
	 vala = 10;  //flag invalid keystroke
  }

  if((vala >= 0) && (vala <= 5))
  {

	 if (M201->tScan == 0)
	 {
		clrscr();
		gotoxy(10,12);
		printf("Channel number:    %d",vala);

		SetWin32CursorOn();
		gotoxy(10,14);
		printf ("Enter external control code, 0 - 15 ");
		#ifdef WIN32
		getch();    /////.new.
		#endif
		fgets(tmpbuf,80,stdin);
		sscanf(tmpbuf,"%d",&M201->subChan);
		SetWin32CursorOff();

		M201->subChan = (short)fmod((double)M201->subChan,16.0);
		M201->channel =(short)vala;
		M201->singleChan =(short)((vala<<4) +  M201->subChan);
	 }
	 else
	 {
		M201->chan[vala] = GetChannel(vala);
	 }

  }

  if(vala == 6)
  {
	 CalcScanTime(M201);
  }

  if(vala == 7)
  {

	 if (M201->tScan == 0)
	 {
		M201->tScan = 1;
	 }
	 else if (M201->tScan == 1)
	 {
		M201->tScan = 2;
	 }
	 else
	 {
		M201->tScan = 0;
	 }

	 if (M201->gain != 1 && M201->tScan == 2)
	 {
		M201->tScan = 0;
	 }

  }

  if(Ch == 13)
  {

	 if(M201->numChannels == 0)
	 {
		printf("You must enter control codes first, press any key to continue");
		getch();
		goto SCANMENU1;

	 }


	 i = CheckScanTime(M201, &maxTime, &minTime);

	 if (i==2)
	 {
		 printf("Reduced averaging to %d- Press any key to continue",M201->average);
		 getch();
	 }
	 else if(!i)
	 {
		clrscr();
		gotoxy(10,15);
		printf("Scan time out of limits, press any key to continue");
		getch();
		CalcScanTime(M201);
	 }

	 return 1;
  }

  if(Ch == 27)
  {
	 return 0;
  }

  goto SCANMENU1;

}


/****************************************************************************
Because all aspects of the scanning function of the Model 201 is programmable,
it can be difficult to determine what the optimal settings are for any
particular task. Also, if the A/D is not performing as intended, it can be
hard to determine the cause. An overview of how to set up the 201 from a
programmer's perspective follows.

The A/D is constantly performing conversions at DATARATE. This rate is set in
the mode registers of the A/D chip itself. Slower DATARATEs provide lower
noise, higher DATARATEs allow higher frequencies to be seen, and allow more
data points to be taken in the same time period.

For higher frequency applications the DATARATE should be set for the
appropriate low-pass filter cutoff frequency. The cutoff frequency is
approximately one quarter the DATARATE.

For DC and very low frequency applications, you would normally select the
lowest possible DATARATE. That lowest rate is determined by all the
programmable scanning variables - scan type, the number of channels scanned,
averaging, filter setting, word length and BAUD rate. Even though the
DATARATE is actually sent to the A/D before the rest of the scanning details
during the sign-on process, it should be calculated last. Details for Model
201 follow:


  M201.baud           = 0 for 9600, 1 for 4800, etc.
  M201.numChannels    = number of channels to scan
  M201.average        = number of conversions to average
								(1, 2, 4, 8, 16, 32 etc)
  M201.wordCnt        = number of bytes in conversion ( 2 or 3)
  M201.filter;		  = 0 = 4 Hz, 1 = 40 Hz, 2 = 400 Hz filter

  For the Model 201, fTime is based on the M201.filter setting:
	 0 (4 Hz), fTime = 4.5 seconds
	 1 (40 Hz), fTime = .56 seconds
	 2 (400 Hz), fTime = .036 seconds

Here, we have solved for the minimum sample rate given the scan interval:

  minSampleRate = (M201.average + 2) / ((M201.scanInterval - (M201.wordCnt *
				 .002333) * 2 ^ M201.baud) / M201.numChannels - (fTime * .82));

A 5 percent margin should provide comfort. The following line would take
care of that:

  M201.rate = 1.05 * minSampleRate;
****************************************************************************/
short CalcScanTime(T_M201 far *M201)	//select the scan interval
{
double minTime, maxTime;
char tmpbuf[80];
short i;

  clrscr();

CALCSCANTIME1:

  if(M201->numChannels == 0)
  {
	 printf("You must enter control codes first, press any key to continue");
	 getch();
	 return 0;
  }

  i = CheckScanTime(M201, &maxTime, &minTime);

  if (i==2)
  {
	 printf("Reduced averaging to %d- Press any key to continue",M201->average);
	 getch();
  }

  SetWin32CursorOn();
  gotoxy(10,10);
  printf("Enter scan time ");
  printf("%10.4lf  - ", minTime);
  printf( "%12.4lf  seconds  ", maxTime);
  #ifdef WIN32
  getch();    /////.new.
  #endif
  fgets(tmpbuf,80,stdin);
  sscanf(tmpbuf,"%lf",&M201->scanInterval);
  SetWin32CursorOff();

  i = CheckScanTime(M201, &maxTime, &minTime);

  if (i==2)
  {
	 printf("Reduced averaging to %d- Press any key to continue",M201->average);
	 getch();
  }
  else if(!i)
  {
	 clrscr();
	 gotoxy(10,15);
	 printf("Scan time out of limits, press any key to continue");
	 getch();
	 goto CALCSCANTIME1;
  }

  return 1;
}

void SelectRate(T_M201 far *M201)    //select the sampling rate for the A/D
{
char tmpbuf[80];

  SetWin32CursorOn();
  gotoxy(1,24);
  printf("            Enter sample rate between 10 and 1027  ");
  #ifdef WIN32
  getch();    /////.new.
  #endif
  fgets(tmpbuf,80,stdin);
  sscanf(tmpbuf,"%f",&M201->rate);
  SetWin32CursorOff();

  if (M201->rate > 1027)
  {
	 M201->rate = 1027;
  }
  else if (M201->rate < 10)
  {
	 M201->rate = 10;
  }

  SendADMode(M201);

  SysCal(M201);

  SetChanCode(M201);

  gotoxy(1,24);
  printf("                                                          ");
}

short ReadConfig(T_M201 far *M201,	//read 201EWIN.CFG file from disk in current directory
					  short *localAvg)
{
char tmpbuf[80];
FILE *rCfg;

	if ((rCfg = fopen("201ewin.cfg", "rt")) != NULL)
	{
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%d",&M201->TXQUEUESIZE);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%d",&M201->RXQUEUESIZE);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%d",&M201->baud);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%d",&M201->comPort);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%lf",&M201->scanInterval);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%d",&M201->numChannels);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%d",&M201->average);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%d",&M201->tScan);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%d",&M201->chan[0]);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%d",&M201->chan[1]);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%d",&M201->chan[2]);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%d",&M201->chan[3]);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%d",&M201->chan[4]);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%d",&M201->chan[5]);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%d",&M201->singleChan);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%d",&M201->channel);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%d",&M201->subChan);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%d",&M201->wordCnt);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%d",&M201->biPolar);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%d",&M201->filter);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%d",&M201->gain);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%f",&M201->rate);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%d",&M201->modeRegLo);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%d",&M201->modeRegMid);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%d",&M201->modeRegHi);
	  fgets(tmpbuf,80,rCfg);
	  sscanf(tmpbuf,"%d",localAvg);
	  fclose(rCfg);
	  return 1;
	}
	else
	{
	  M201->TXQUEUESIZE = 1;
	  M201->RXQUEUESIZE = 1024;
	  M201->baud = 0;
	  M201->comPort = 0;
	  M201->filter = 2;
	  M201->gain = 1;
	  M201->rate = 10;
	  M201->wordCnt = 3;
	  M201->biPolar = 1;
	  M201->average = 1;
	  M201->scanInterval = 2;
	  M201->numChannels = 6;
	  M201->chan[0]= 0;
	  M201->chan[1]= 0;
	  M201->chan[2]= 0;
	  M201->chan[3]= 0;
	  M201->chan[4]= 0;
	  M201->chan[5]= 0;
	  M201->singleChan = 0;
	  M201->tScan = 1;
	  M201->channel = 0;
	  M201->subChan = 0;
	  (*localAvg) = 1;
	  return 0;
	}

}


short WriteConfig(T_M201 far *M201,	//write 201EWIN.CFG file to disk in current directory
						short *localAvg)
{
FILE *wCfg;

  if ((wCfg = fopen("201ewin.cfg", "wt")) != NULL)
  {
	 fprintf(wCfg,"%d\n",M201->TXQUEUESIZE);
	 fprintf(wCfg,"%d\n",M201->RXQUEUESIZE);
	 fprintf(wCfg,"%d\n",M201->baud);
	 fprintf(wCfg,"%d\n",M201->comPort);
	 fprintf(wCfg,"%lf\n",M201->scanInterval);
	 fprintf(wCfg,"%d\n",M201->numChannels);
	 fprintf(wCfg,"%d\n",M201->average);
	 fprintf(wCfg,"%d\n",M201->tScan);
	 fprintf(wCfg,"%d\n",M201->chan[0]);
	 fprintf(wCfg,"%d\n",M201->chan[1]);
	 fprintf(wCfg,"%d\n",M201->chan[2]);
	 fprintf(wCfg,"%d\n",M201->chan[3]);
	 fprintf(wCfg,"%d\n",M201->chan[4]);
	 fprintf(wCfg,"%d\n",M201->chan[5]);
	 fprintf(wCfg,"%d\n",M201->singleChan);
	 fprintf(wCfg,"%d\n",M201->channel);
	 fprintf(wCfg,"%d\n",M201->subChan);
	 fprintf(wCfg,"%d\n",M201->wordCnt);
	 fprintf(wCfg,"%d\n",M201->biPolar);
	 fprintf(wCfg,"%d\n",M201->filter);
	 fprintf(wCfg,"%d\n",M201->gain);
	 fprintf(wCfg,"%f\n",M201->rate);
	 fprintf(wCfg,"%d\n",M201->modeRegLo);
	 fprintf(wCfg,"%d\n",M201->modeRegMid);
	 fprintf(wCfg,"%d\n",M201->modeRegHi);
	 fprintf(wCfg,"%d\n",(*localAvg));
	 fclose(wCfg);
	 return 1;
  }

  return 0;

}

void SetWindowToDesktop(void)  //make window the same size as the desktop
{
#ifdef WIN16
RECT rect;

  clrscr();
  GetWindowRect(GetDesktopWindow(), &rect);
  SetWindowPos(GetActiveWindow(), (HWND) NULL, 0, 0,
					rect.right, rect.bottom,
					SWP_NOZORDER | SWP_NOACTIVATE);
#else
  _setcursortype(_NOCURSOR);
#endif
}

void SetWin32CursorOn(void)		//turns on cursor in Win32 console mode
{
#ifdef WIN32
  _setcursortype(_NORMALCURSOR);
#endif
}

void SetWin32CursorOff(void)		//turns off cursor in Win32 console mode
{
#ifdef WIN32
  _setcursortype(_NOCURSOR);
#endif
}
